<?php

define('INDEX_TOO_LONG', 1);
define('INDEX_ALREADY_EXISTS', 2);
define('INDEX_ILLEGAL_TYPE', 3);

/**
 * Implements hook_drush_command().
 */
function mtools_drush_command() {
  // the key in the $items array is the name of the command.
  $items = array();
  $items['mt-clear-vocabulary'] = array(
    'description' => "Remove all terms for a particular vocabulary.",
    'drupal dependencies' => array('taxonomy'),
    'arguments' => array(
      'vocabulary' => 'The vocabulary machine name to clear.',
    ),
    'examples' => array(
      'clear-vocabulary example_vocabulary' => 'Remove all terms from the "example_vocabulary" vocabulary.',
    ),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
  );
  $items['mt-delete-all-nodes'] = array(
    'description' => "Delete all content in the Drupal database. Delete *all* nodes.",
   'bootstrap' => DRUSH_BOOTSTRAP_DRUSH_FULL,
  );
  $items['mt-delete-node-type'] = array(
    'description' => "Delete all the nodes of a specific type in the Drupal database.",
    'arguments' => array(
      'type' => 'The node type to clear.',
    ),
   'bootstrap' => DRUSH_BOOTSTRAP_DRUSH_FULL,
  );
  $items['mt-delete-all-users'] = array(
    'description' => 'Delete all users except for uid 1',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH_FULL,
  );
  $items['mt-delete-unused-accounts'] = array(
    'description' => 'Deletes account that have no content or comments associated with them or that have not been used or verified.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH_FULL,
  );
  $items['mt-add-indexes'] = array(
    'description' => 'Adds indexes to all custom content fields.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH_FULL,
  );
  $items['mt-field-info'] = array(
    'description' => 'Output field and instance information for a given field (debugging).',
    'arguments' => array(
      'field' => 'The field to get and output information on. (used for etype debugging).',
    ),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH_FULL,
  );

  return $items;
}

/**
 * Implements drush_HOOK_COMMAND()
 * Handler to delete all users with a uid greater than 1. The first user is a special
 * Drupal case and we never want to delete that.
 */
function drush_mtools_mt_delete_all_users() {
  // Get the complete list of uid's > 1
  $uids = array();
  $result = db_query("SELECT uid FROM {users} WHERE uid > 1");
  while($udata = $result->fetchObject()) {
    $uids[] = $udata->uid;
  }
  drush_log('Removing ' . count($uids) . ' users.', 'ok');
  user_delete_multiple($uids);
  drush_log(count($uids) . ' users removed.' ,'ok');
}

/**
 * Implements drush_HOOK_COMMAND()
 * Handler to delete all users with a uid greater than 1. The first user is a special
 * Drupal case and we never want to delete that.
 */
function drush_mtools_mt_delete_unused_accounts() {
  // Get the complete list of uid's > 1 that have no nodes or comments assigned to them
  // Merge with a list that have  not be accessed, verified, or logged in.
  $uids = array();
  $result = db_query("SELECT u.uid AS user_user_id,
                             n.uid AS node_user_id, 
                             c.uid AS comment_user_id
                      FROM {users} u
                      LEFT JOIN {node} n ON n.uid=u.uid
                      LEFT JOIN {comment} c ON c.uid=u.uid
                      WHERE n.uid IS NULL 
                        AND c.uid IS NULL
                        AND u.uid > 1");
  while($udata = $result->fetchObject()) {
    $uids[$udata->user_user_id] = $udata->user_user_id;
  }
  $result = db_query("SELECT u.uid AS user_user_id
                      FROM {users} u
                      WHERE (u.access=:access OR u.login=:login OR u.status=:status)",
               array(':access' => 0, ':login' => 0, ':status' => 0));
  while($udata = $result->fetchObject()) {
    $uids[$udata->user_user_id] = $udata->user_user_id;
  }
  drush_log('Removing ' . count($uids) . ' users.', 'ok');
  user_delete_multiple($uids);
  drush_log(count($uids) . ' users removed.' ,'ok');
}

/**
 * Implements drush_HOOK_COMMAND()
 *
 * Clear everything from our nodes table relative to the content type
 */
function drush_mtools_mt_delete_node_type($type = NULL) {
  // If a node type was not provided, then give a list of node types for the user to
  // select from
  if (empty($type)) {
    $types = node_type_get_types();
    $options = array('all' => 'All');
    foreach ($types as $type=>$object) {
      $options[$type] = $object->name;
    }
    $choice = drush_choice($options, 'Which node type do you wish to clear?'); 
    if ($choice == 'all') {
      foreach($options as $key => $option) {
        if ($key == 'all') {
          continue;
        }
        _mtools_delete_node($key);
      }
    } 
    else {
      _mtools_delete_node($choice);
    }
  }
}    

/**
 * Implements drush_HOOK_COMMAND()
 * Clear everything from our nodes table regardless of content type. We also clear out
 * the taxonomy terms.
 * TODO: Make the taxonomy term deletion an argument and not required as it may not always
 *       be required due to import routines
 */
function drush_mtools_mt_delete_all_nodes() {
  /* we're going to list the deleted nodes by title */
  $result = db_query("SELECT nid,title FROM {node} ORDER BY nid");

  /* get the number of nodes */
  $node_count = $result->rowCount();
  drush_log("Deleting " . $node_count . " nodes", 'ok');

  /* iterate through the nodes, reporting on their delete status */
  $count = 1;
  while($ndata = $result->fetchObject()) {
    node_delete($ndata->nid);
    drush_log('Deleted: ' . $count . ' of ' . $node_count . ': ' . $ndata->title, 'ok');
    $count++;
  }
  
  drush_log("\n\nDeleting Taxonomy Terms", 'ok');
  
  /* because we're deleting, we need to clear out all of the taxonomies too */
  /* in the event they're being used by an import routine (bondware) */
  $vocabularies = taxonomy_get_vocabularies();
  foreach($vocabularies as $vid => $vocabulary) {
    $terms = taxonomy_get_tree($vid);
    foreach($terms as $term) { 
      drush_log("Deleting: " . $term->name, 'ok');
      taxonomy_term_delete($term->tid);
    }
  }
}

/**
 * Implements drush_HOOK_COMMAND()
 * Remove all terms from a specific taxonomy vocabulary
 */
function drush_mtools_mt_clear_vocabulary($vocabulary = FALSE) {
  // If the vocabulary was not specified, give them a list to choose from, the first on
  // the list being "all"
  if (empty($vocabulary) || $vocabulary == FALSE) {
    $options = array('all' => 'All');
    $vocabularies = taxonomy_get_vocabularies();
    foreach($vocabularies as $vocabulary) {
      $options[$vocabulary->machine_name] = $vocabulary->name;
    }
    $choice = drush_choice($options, 'Which vocabulary do you wish to clear?'); 
    if ($choice && array_key_exists($choice, $options)) {
      if ($choice == 'all') {
        foreach($options as $key => $option) {
          if ($key == 'all') {
            continue;
          }
          $vocabulary_object = taxonomy_vocabulary_machine_name_load($key);
          _mtools_delete_vocabulary($vocabulary_object);
        }
      } 
      else {
        $vocabulary_object = taxonomy_vocabulary_machine_name_load($choice);
        _mtools_delete_vocabulary($vocabulary_object);
      } 
    }
    else {
      drush_set_error('Unable to locate any vocabulary named ' . $choice);
    }
  }
  else {
    if ($vocabulary_object = taxonomy_vocabulary_machine_name_load($vocabulary)) {
      _mtools_delete_vocabulary($vocabulary_object);
    } 
    else {
      drush_set_error('Unable to locate any vocabulary named ' . $vocabulary);
    }
  }
}

/**
 * Implements hook_highwire_add_indexes().
 *
 * Add indexes to CCK fields
 */
function drush_mtools_mt_add_indexes() {
 
  $message = array(
    '',
    'Unable to index field, length of index would be too long.',
    'Unable to index field, index already exists.',
    'Unable to index field, not a valid field for indexing.',
  );
 
 
  $fields = array_keys(field_info_fields());
  $field_data_prefix = 'field_data_';
  foreach ($fields as $field) {
    $table = $field_data_prefix . $field;
    $data = db_query("SHOW COLUMNS FROM $table", array());

    while ($column = $data->fetchAssoc()) {
      // Initialize our condition
      $index_condition = NULL;
    
      // Check to see if the amount of characters we're trying to index is > 255 for varchar      
      $matches = array();
      if (stristr($column['Type'], 'varchar')) {
        preg_match_all('/\((.*?)\)/', $column['Type'], $matches);
        $length = $matches[1][0];
        if ($length > 255) {
          $index_condition = INDEX_TOO_LONG;
        }
      }
      
      if ($index_condition == NULL && !empty($column['Key'])) {
        $index_condition = INDEX_ALREADY_EXISTS;
      }
 
      if (stristr($column['Type'], 'text') || stristr($column['Type'], 'blob') || stristr($column['Type'], 'longblob') || stristr($column['Type'], 'longtext')) {
        $index_condition = INDEX_ILLEGAL_TYPE;
      }
 
      if (stristr($column['Field'], 'data') || stristr($column['Field'], 'format') || stristr($column['Field'], 'entity_type')) {
        $index_condition = INDEX_ILLEGAL_TYPE;
      }
      
      if ($index_condition == NULL) {
        db_add_index($table, $column['Field'], array($column['Field']));
        drush_log($column['Field'] , 'success');
      }
      else {
        if ($index_condition != INDEX_ALREADY_EXISTS) {
          // I really don't want a ton of output on this especially for a "run once" type
          // of script that would throw a ton of errors on subsequent runs.
          //drush_log($column['Field'] . ': ' . $message[$index_condition], 'error');
        }   
      }
    }
  }
}

/**
 * Implements drush_HOOK_COMMAND()
 */
function drush_mtools_mt_field_info($field) {
  $field_info = field_info_field($field);
  print_r($field_info);
    
  //field_info_instance($entity_type, $field, $bundle);
  //print_r($field_instance);
}


/**
 * Helper function to delete a vocabulary
 *
 * @param object $vocabulary
 *   The Drupal vocabulary object to be deleted
 */
function _mtools_delete_vocabulary($vocabulary) {
  $tree = taxonomy_get_tree($vocabulary->vid);
  foreach($tree as $term) {
    if (taxonomy_term_delete($term->tid) == SAVED_DELETED) {
      drush_log($term->name . ' successfully deleted.', 'ok');
    } else {
      drush_log($term->name . ' could not be deleted.', 'error');
    }
  }
}

/**
 * Helper function to delete content from a node type
 *
 * @param string $type
 *   The machine node type name to clear the content from
 */
function _mtools_delete_node($type) {
  if (!empty($type)) {
    // We're going to list the deleted nodes by title
    $result = db_query("SELECT nid,title FROM {node} WHERE type=:type ORDER BY nid", array(':type' => $type));

    // Get the number of nodes
    $node_count = $result->rowCount();
    drush_log("Deleting " . $node_count . " nodes", 'ok');

    // Iterate through the nodes, reporting on their delete status
    $count = 1;
    while($ndata = $result->fetchObject()) {
      node_delete($ndata->nid);
      drush_log('Deleted: ' . $count . ' of ' . $node_count . ': ' . $ndata->title, 'ok');
      $count++;
    }
  }
}

